namespace eval ::mailroom {
  variable db_version 1
  
  variable db_schema "
    -- General configuraton infomation is stored in the var() table.
    -- var.name1 and var.name2 are keys.  var.value is the data.
    --
    create table var (
      name1 text,
      name2 text,
      value any,
      primary key (name1, name2)
    );
    insert into var (name1,name2,value) values ('internal','dbver',$db_version);
    
    -- The body of email messages (which can be very large) are stored in 
    -- the BLOB table separate from the main MSG table.  This allows us to
    -- do scans of the MSG table without having to skip over the huge
    -- blobs of email content.  
    --
    create table blob (
      blobid integer primary key,         -- blob id.  Same as msgid
      data blob                           -- original content of the message
    );
    
    -- One entry in this table for each email message.
    --
    create table msg (
      msgid            integer primary key, -- unique integer message id
      bucket           text,                -- inbox, 
      received         datetime,            -- when message first received
      sent             datetime,            -- when message was sent (according to it)
      size             integer,             -- size of the blob
      attachment_count integer,             -- number of attachments
      from_userid      integer,             -- who the message is from
      subject          text,                -- subject text
      is_read          integer              -- has the message been read yet?
    );
    create index msg_idx_received on msg (received, subject);
    create index msg_idx_subj on msg (subject);
    create index msg_idx_xfrom on msg (from_userid);
    
    -- Message IDs are used for merging databases.  They are stored
    -- separately from the MSG table for efficiency.
    --
    create table msg_uuid (
      msgid integer primary key,          -- The message
      uuid text unique                    -- Its universally unique ID
    );
    
    -- Messages that need to be sent.
    --
    create table msg_send_queue (
      msgid integer primary key           -- The message
    );
    
    -- Users that we have received email from or that we have sent email
    -- to or who have appeared on the To: or Cc: lists of emails we have
    -- received.
    --
    create table user (
      userid integer primary key,         -- Linkage to MSG.USERID
      address text collate nocase unique, -- userid@domain.name
      domain text,                        -- domain.name
      fulladdr text,                      -- 'Human Name' <userid@domain.name>
      recent datetime,                    -- time most recently seen
      to_count integer default 0,         -- number of times in To: or Cc: on outgoing messages
      from_count integer default 0,       -- number of times in From: on incoming messages
      spam_count integer default 0,       -- number of times associated message was spam
      edited default 0,                   -- has the user edited this user? If 1, do not auto-update
      importance default 3                -- 1=low, 3=normal, 5=high
    );
    
    -- Full-text index on message content
    --
    create virtual table email using fts3 (
       from_who,          -- who the message is from
       to_who,            -- who the message is to
       subject,           -- subject of the message
       body,              -- content of the message
       tokenize porter    -- Use the Porter stemmer
    );
    "
  
  variable memdb_schema "
    -- For mapping mime elements in the mime display into information
    -- needed to extract the mime.  This is used in the 'MIME' display
    -- format.
    --
    create temp table if not exists mime (
      lineno integer,   -- Line number of 'view' or 'save' hyperlink
      partid text,      -- MIME subcomponent index
      filename text,    -- MIME filename (if any)
      content text      -- MIME content type
    );
    "
}
